use structopt::StructOpt;
use std::{
    path::PathBuf,
    fs::File,
    io::prelude::*,
    boxed::Box,
    error::Error,
    sync::Arc,
};
use serde::Deserialize;
use sqlx::{
    prelude::*,
    sqlite::{
        SqliteConnection,
        SqliteConnectOptions,
    },
};
use tokio::{
    sync::RwLock,
};

const DEFAULT_DATABASE_PATH: &'static str = "/var/lib/noise-server/noise-server.sqlite";

#[derive(Debug, StructOpt)]
struct Options {
    #[structopt(short, long, default_value = "/etc/noise-server/config.toml")]
    config: PathBuf,

    #[structopt(short, long, default_value = DEFAULT_DATABASE_PATH)]
    database_path: PathBuf,
}

#[derive(Debug, Deserialize)]
#[serde(default)]
struct Config {
    database_path: PathBuf,
}

impl Default for Config {
    fn default() -> Self {
        Config {
            database_path: DEFAULT_DATABASE_PATH.into(),
        }
    }
}

#[tokio::main]
async fn main() -> Result<(), Box<dyn Error>> {
    let clap_matches = Options::clap().get_matches();
    let options = Options::from_clap(&clap_matches);
    
    let mut config = get_config(&options)?;
    
    if let Some(db_path) = clap_matches.value_of("database_path") {
        config.database_path = db_path.into();
    }

    #[cfg(debug_assertions)]
    println!("{:?}", options);

    #[cfg(feature = "sqlite")]
    let sqlite_connection = Arc::new(
        RwLock::new(
            connect_sqlite(&config.database_path).await?
        )
    );

    Ok(())
}

/// Prepare sqlite database connection
#[cfg(feature = "sqlite")]
async fn connect_sqlite(path: &PathBuf) -> sqlx::Result<SqliteConnection> {
    let connect_options = SqliteConnectOptions::new()
        .filename(&path)
        .create_if_missing(true);
    SqliteConnection::connect_with(&connect_options).await
}

#[cfg(feature = "sqlite")]
async fn load_sqlite_template(){
    
}

fn get_config(options: &Options) -> std::io::Result<Config> {
    let mut config_file = File::open(&options.config)?;
    let mut config_string = String::new();
    config_file.read_to_string(&mut config_string)?;
    
    toml::from_str(&config_string)
        .map_err(|err| std::io::Error::new(
            std::io::ErrorKind::InvalidData, 
            err
        )
    )
}